using data.Domain.UseCase;
using data.RemoteData.RemoteDataBase;
using data.Repository;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;

var builder = WebApplication.CreateBuilder(args);

//      
builder.Services.AddDbContext<RemoteDatabaseContext>(); //   

//    UseCase
builder.Services.AddScoped<IUserRepository, SQLUserRepositoryImpl>()
                .AddScoped<IPresenceRepository, SQLPresenceRepositoryImpl>()
                .AddScoped<IGroupRepository, SQLGroupRepositoryImpl>()
                .AddScoped<UseCaseGeneratePresence>()
                .AddScoped<GroupUseCase>();

//  
builder.Services.AddControllers();

//  Swagger  API 
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen();

//  
builder.Services.AddLogging(logging =>
{
    logging.ClearProviders(); //   
    logging.AddConsole(); //    
    logging.AddDebug(); //   
});

var app = builder.Build();

//  Swagger UI  
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI();
}

//  middleware   
app.UseMiddleware<RequestLoggingMiddleware>();

//  Middleware
app.UseHttpsRedirection();
app.MapControllers();

app.Run();

// Middleware   
public class RequestLoggingMiddleware
{
    private readonly RequestDelegate _next;
    private readonly ILogger<RequestLoggingMiddleware> _logger;

    public RequestLoggingMiddleware(RequestDelegate next, ILogger<RequestLoggingMiddleware> logger)
    {
        _next = next;
        _logger = logger;
    }

    public async Task InvokeAsync(HttpContext context)
    {
        //   
        _logger.LogInformation(" : {Method} {Path}  {Time}",
            context.Request.Method, context.Request.Path, DateTime.UtcNow);

        //     
        await _next(context);

        //   
        _logger.LogInformation(" : {StatusCode}  {Method} {Path}  {Time}",
            context.Response.StatusCode, context.Request.Method, context.Request.Path, DateTime.UtcNow);
    }
}
